(*
** $Id: lauxlib.h,v 1.52 2002/08/08 20:08:41 roberto Exp $
** Auxiliary functions for building Lua libraries
** See Copyright Notice in lua.h
*)
(*
** Translated to pascal by Lavergne Thomas and Marcus Aurelius Farias
** Notes:
**    - Pointers type was prefixed with 'P'
** Bug reports:
**    - thomas.lavergne@laposte.net
**    In French or in English
**
**    - marcus.cf@bol.com.br
**    In Portuguese or in English
*)
unit lauxlib;

interface

uses
  Lua;

type
  luaL_reg = record
    name: PChar;
    func: lua_CFunction;
  end;
  PluaL_reg = ^luaL_reg;
  PPChar = ^PChar;

procedure luaL_openlib(L: Plua_State; libname: PChar; lr: PluaL_reg; nup: Integer); cdecl;
function luaL_getmetafield(L: Plua_State; obj: Integer; e: PChar): Integer; cdecl;
function luaL_callmeta(L: Plua_State; obj: Integer; e: PChar): Integer; cdecl;
function luaL_typerror(L: Plua_State; narg: Integer; tname: PChar): Integer; cdecl;
function luaL_argerror(L: Plua_State; numarg: Integer; extramsg: PChar): Integer; cdecl;
function luaL_checklstring(L: Plua_State; numArg: Integer; len: Psize_t): PChar; cdecl;
function luaL_optlstring(L: Plua_State; numArg: Integer; def: PChar; len: Psize_t): PChar; cdecl;
function luaL_checknumber(L: Plua_State; numArg: Integer): lua_Number; cdecl;
function luaL_optnumber(L: Plua_State; nArg: Integer; def: lua_Number): lua_Number; cdecl;

procedure luaL_checkstack(L: Plua_State; sz: Integer; msg: PChar); cdecl;
procedure luaL_checktype(L: Plua_State; narg, t: Integer); cdecl;
procedure luaL_checkany(L: Plua_State; narg: Integer); cdecl;

function luaL_newmetatable (L: Plua_State; tname: PChar): Integer; cdecl;
procedure luaL_getmetatable (L: Plua_State; tname: PChar); cdecl;
function luaL_checkudata (L: Plua_State; ud: Integer; tname: PChar): Pointer; cdecl;

procedure luaL_where(L: Plua_State; lvl: Integer); cdecl;
function luaL_error(L: Plua_State; fmt: PChar; argsup: array of pointer): Integer; cdecl;

function luaL_findstring(st: PChar; list: PPChar): Integer; cdecl;

function luaL_ref(L: Plua_State; t: Integer): Integer; cdecl;
procedure luaL_unref(L: Plua_State; t, ref: Integer); cdecl;

function luaL_getn(L: Plua_State; t: Integer): Integer; cdecl;
procedure luaL_setn(L: Plua_State; t, n: Integer); cdecl;

function luaL_loadfile(L: Plua_State; filename: PChar): Integer; cdecl;
function luaL_loadbuffer(L: Plua_State; buff: PChar; sz: size_t; name: PChar): Integer; cdecl;

(*
** ===============================================================
** some useful macros
** ===============================================================
*)

procedure luaL_argcheck(L: Plua_State; cond: Boolean; numarg: Integer; extramsg: PChar);
function luaL_checkstring(L: Plua_State; n: Integer): PChar;
function luaL_optstring(L: Plua_State; n: Integer; d: PChar): PChar;
function luaL_checkint(L: Plua_State; n: Integer): Integer;
function luaL_checklong(L: Plua_State; n: Integer): LongInt;
function luaL_optint(L: Plua_State; n: Integer; d: lua_Number): Integer;
function luaL_optlong(L: Plua_State; n: Integer; d: lua_Number): LongInt;

(*
** {======================================================
** Generic Buffer manipulation
** =======================================================
*)

const
  LUAL_BUFFERSIZE = 4096;

type
  luaL_Buffer = record
    p: PChar;
    level: Integer;
    L: Plua_State;
    buffer: array [0..LUAL_BUFFERSIZE - 1] of Char;
  end;
  PluaL_Buffer = ^luaL_Buffer;

procedure luaL_putchar(B: PluaL_Buffer; c: Char);

procedure luaL_addsize(B: PluaL_Buffer; n: Integer);

procedure luaL_buffinit(L: Plua_State ; B: PluaL_Buffer); cdecl;
function luaL_prepbuffer(B: PluaL_Buffer): PChar; cdecl;
procedure luaL_addlstring(B: PluaL_Buffer; const s: PChar; l: size_t); cdecl;
procedure luaL_addstring(B: PluaL_Buffer; const s: PChar); cdecl;
procedure luaL_addvalue(B: PluaL_Buffer); cdecl;
procedure luaL_pushresult(B: PluaL_Buffer); cdecl;

(*
** Compatibility macros
*)

function lua_dofile(L: Plua_State; filename: PChar): Integer; cdecl;
function lua_dostring(L: Plua_State; str: PChar): Integer; cdecl;
function lua_dobuffer(L: Plua_State; buff: PChar; size: Integer; name: PChar): Integer; cdecl;

function luaL_check_lstr(L: Plua_State; numArg: Integer; len: Psize_t): PChar; cdecl;
function luaL_opt_lstr(L: Plua_State; numArg: Integer; def: PChar; len: Psize_t): PChar; cdecl;
function luaL_check_number(L: Plua_State; numArg: Integer): lua_Number; cdecl;
function luaL_opt_number(L: Plua_State; nArg: Integer; def: lua_Number): lua_Number; cdecl;
procedure luaL_arg_check(L: Plua_State; cond: Boolean; numarg: Integer; extramsg: PChar);
function luaL_check_string(L: Plua_State; n: Integer): PChar;
function luaL_opt_string(L: Plua_State; n: Integer; d: PChar): PChar;
function luaL_check_int(L: Plua_State; n: Integer): Integer;
function luaL_check_long(L: Plua_State; n: Integer): LongInt;
function luaL_opt_int(L: Plua_State; n: Integer; d: lua_Number): Integer;
function luaL_opt_long(L: Plua_State; n: Integer; d: lua_Number): LongInt;

implementation

procedure luaL_openlib(L: Plua_State; libname: PChar; lr: PluaL_reg; nup: Integer); cdecl; external LUA_LIB_NAME;
function luaL_getmetafield(L: Plua_State; obj: Integer; e: PChar): Integer; cdecl; external LUA_LIB_NAME;
function luaL_callmeta(L: Plua_State; obj: Integer; e: PChar): Integer; cdecl; external LUA_LIB_NAME;
function luaL_typerror(L: Plua_State; narg: Integer; tname: PChar): Integer; cdecl; external LUA_LIB_NAME;
function luaL_argerror(L: Plua_State; numarg: Integer; extramsg: PChar): Integer; cdecl; external LUA_LIB_NAME;
function luaL_checklstring(L: Plua_State; numArg: Integer; len: Psize_t): PChar; cdecl; external LUA_LIB_NAME;
function luaL_check_lstr(L: Plua_State; numArg: Integer; len: Psize_t): PChar; cdecl; external LUA_LIB_NAME name 'luaL_checklstring';
function luaL_optlstring(L: Plua_State; numArg: Integer; def: PChar; len: Psize_t): PChar; cdecl; external LUA_LIB_NAME;
function luaL_opt_lstr(L: Plua_State; numArg: Integer; def: PChar; len: Psize_t): PChar; cdecl; external LUA_LIB_NAME name 'luaL_optlstring';
function luaL_checknumber(L: Plua_State; numArg: Integer): lua_Number; cdecl; external LUA_LIB_NAME;
function luaL_check_number(L: Plua_State; numArg: Integer): lua_Number; cdecl; external LUA_LIB_NAME name 'luaL_checknumber';
function luaL_optnumber(L: Plua_State; nArg: Integer; def: lua_Number): lua_Number; cdecl; external LUA_LIB_NAME;
function luaL_opt_number(L: Plua_State; nArg: Integer; def: lua_Number): lua_Number; cdecl; external LUA_LIB_NAME name 'luaL_optnumber';

procedure luaL_checkstack(L: Plua_State; sz: Integer; msg: PChar); cdecl; external LUA_LIB_NAME;
procedure luaL_checktype(L: Plua_State; narg, t: Integer); cdecl; external LUA_LIB_NAME;
procedure luaL_checkany(L: Plua_State; narg: Integer); cdecl; external LUA_LIB_NAME;

function luaL_newmetatable (L: Plua_State; tname: PChar): Integer; cdecl; external LUA_LIB_NAME;
procedure luaL_getmetatable (L: Plua_State; tname: PChar); cdecl; external LUA_LIB_NAME;
function luaL_checkudata (L: Plua_State; ud: Integer; tname: PChar): Pointer; cdecl; external LUA_LIB_NAME;

procedure luaL_where(L: Plua_State; lvl: Integer); cdecl; external LUA_LIB_NAME;
function luaL_error(L: Plua_State; fmt: PChar; argsup: array of pointer): Integer; cdecl; external LUA_LIB_NAME;

function luaL_findstring(st: PChar; list: PPChar): Integer; cdecl; external LUA_LIB_NAME;

function luaL_ref(L: Plua_State; t: Integer): Integer; cdecl; external LUA_LIB_NAME;
procedure luaL_unref(L: Plua_State; t, ref: Integer); cdecl; external LUA_LIB_NAME;

function luaL_getn(L: Plua_State; t: Integer): Integer; cdecl; external LUA_LIB_NAME;
procedure luaL_setn(L: Plua_State; t, n: Integer); cdecl; external LUA_LIB_NAME;

function luaL_loadfile(L: Plua_State; filename: PChar): Integer; cdecl; external LUA_LIB_NAME;
function luaL_loadbuffer(L: Plua_State; buff: PChar; sz: size_t; name: PChar): Integer; cdecl; external LUA_LIB_NAME;

procedure luaL_argcheck(L: Plua_State; cond: Boolean; numarg: Integer; extramsg: PChar);
begin
  if not cond then luaL_argerror(L, numarg, extramsg);
end;

procedure luaL_arg_check(L: Plua_State; cond: Boolean; numarg: Integer; extramsg: PChar);
begin
  luaL_argcheck(L, cond, numarg, extramsg);
end;

function luaL_checkstring(L: Plua_State; n: Integer): PChar;
begin
  result := luaL_checklstring(L, n, nil);
end;

function luaL_check_string(L: Plua_State; n: Integer): PChar;
begin
  result := luaL_checkstring(L, n);
end;

function luaL_optstring(L: Plua_State; n: Integer; d: PChar): PChar;
begin
  result := luaL_optlstring(L, n, d, nil);
end;

function luaL_opt_string(L: Plua_State; n: Integer; d: PChar): PChar;
begin
  result := luaL_optstring(L, n, d);
end;

function luaL_checkint(L: Plua_State; n: Integer): Integer;
begin
  result := Integer(Trunc(luaL_checknumber(L, n)));
end;

function luaL_check_int(L: Plua_State; n: Integer): Integer;
begin
  result := luaL_checkint(L, n);
end;

function luaL_checklong(L: Plua_State; n: Integer): LongInt;
begin
  result := LongInt(Trunc(luaL_checknumber(L, n)));
end;

function luaL_check_long(L: Plua_State; n: Integer): LongInt;
begin
  result := luaL_checklong(L, n);
end;

function luaL_optint(L: Plua_State; n: Integer; d: lua_Number): Integer;
begin
  result := Integer(Trunc(luaL_opt_number(L, n, lua_Number(d))));
end;

function luaL_opt_int(L: Plua_State; n: Integer; d: lua_Number): Integer;
begin
  result := luaL_optint(L, n, d);
end;

function luaL_optlong(L: Plua_State; n: Integer; d: lua_Number): LongInt;
begin
  result := LongInt(Trunc(luaL_opt_number(L, n, lua_Number(d))));
end;

function luaL_opt_long(L: Plua_State; n: Integer; d: lua_Number): LongInt;
begin
  result := luaL_optlong(L, n, d);
end;

procedure luaL_putchar(B: PluaL_Buffer; c: Char);
begin
  if Cardinal(@(B^.p)) < (Cardinal(@(B^.buffer[0])) + LUAL_BUFFERSIZE) then
    luaL_prepbuffer(B);
  B^.p[1] := c;
  B^.p := B^.p + 1;
end;

procedure luaL_addsize(B: PluaL_Buffer; n: Integer);
begin
  B^.p := B^.p + n;
end;

procedure luaL_buffinit(L: Plua_State ; B: PluaL_Buffer); cdecl; external LUA_LIB_NAME;
function luaL_prepbuffer(B: PluaL_Buffer): PChar; cdecl; external LUA_LIB_NAME;
procedure luaL_addlstring(B: PluaL_Buffer; const s: PChar; l: size_t); cdecl; external LUA_LIB_NAME;
procedure luaL_addstring(B: PluaL_Buffer; const s: PChar); cdecl; external LUA_LIB_NAME;
procedure luaL_addvalue(B: PluaL_Buffer); cdecl; external LUA_LIB_NAME;
procedure luaL_pushresult(B: PluaL_Buffer); cdecl; external LUA_LIB_NAME;

function lua_dofile(L: Plua_State; filename: PChar): Integer; cdecl; external LUA_LIB_NAME;
function lua_dostring(L: Plua_State; str: PChar): Integer; cdecl; external LUA_LIB_NAME;
function lua_dobuffer(L: Plua_State; buff: PChar; size: Integer; name: PChar): Integer; cdecl; external LUA_LIB_NAME;

end.